//
//cl rec2ts.c
//
//converts a .rec file into a .ts file
//saves the header information into a .csv file
//
//bdb - 20060808
//
#include <windows.h>
#include <stdio.h>

#define bitrev16(d)        ((d << 8) & 0xff00) + ((d >> 8) & 0x00ff)
#define bitrev32(d)        ((d << 24) & 0xff000000) + ((d << 8) & 0xff0000) + ((d >> 8) & 0xff00) + ((d >> 24) & 0x00ff)

typedef struct{
    ULONG   magic;              //0xa8   0x00   TFrc
    USHORT  version;            //0xac   0x04   5010    //version
    USHORT  word_ae;            //0xae   0x06   0000
    USHORT  fduration;          //0xb0   0x08   0001    //file duration (mins)???
    USHORT  svcNum;             //0xb2   0x0a   000d        //lcn   (always 0 )
    USHORT  svcType;            //0xb4   0x0c   0000
    USHORT  word_b6;            //0xb8   0x0e   0000
    USHORT  word_b8;            //0xb8   0x10   04b0        //tuner
    USHORT  service_id;         //0xba   0x12   20fa
    USHORT  pmt_pid;            //0xbc   0x14   010e
    USHORT  pcr_pid;            //0xbe   0x16   1ffe
    USHORT  video_pid;          //0xc0   0x18   024e
    USHORT  audio_pid;          //0xc2   0x1a   824f        // | 0x8000
    UCHAR   channel_name[0x1c]; //0xc4   0x1c   More 4
    UCHAR   byte_e0;            //0xe0   0x38   00
    UCHAR   uhf_channel;        //0xe1   0x39   16
    UCHAR   bandwidth;          //0xe2   0x3a   08
    UCHAR   byte_e3;            //0xe3   0x3b   00
    ULONG   frequency;          //0xe4   0x3c   00075ad0
    USHORT  transport_stream_id;//0xe8   0x40   2005
    USHORT  word_ea;            //0xea   0x42   0000
    USHORT  original_network_id;//0xec   0x44   233a
    USHORT  word_ee;            //0xee   0x46   3005
    USHORT  word_f0;            //0xf0   0x48   8000
    USHORT  duration;           //0xf2   0x4a   0123     //epg duration hr:min
    USHORT  word_f4;            //0xf4   0x4c   0000
    USHORT  event_id;           //0xf6   0x4e   793e
    ULONG   startTime;          //0xf8   0x50   d2b9151e
    ULONG   endTime;            //0xfc   0x54   d2b91705
    UCHAR   runningStatus;      //0x100  0x58   04
    UCHAR   eventName_length;   //0x101  0x59   03
    UCHAR   parental_rating;    //0x102  0x5a   00
    UCHAR   eventName_description[0x100];   //0x103   0x5b GBHThe only ray of ...
    UCHAR   padding[0x400];   //0x103   0x5b GBHThe only ray of ...
}type_rec_header;

int extract_mjd(USHORT mjd, UCHAR *day, UCHAR *month, USHORT *year)
{
    int yy,mm,k;

    yy = (int)((mjd - 15078.2) / 365.25);
    mm = (int) ((mjd - 14956.1 - (int) (yy * 365.25)) / 30.6001);
    k = ((mm == 14) || (mm == 15)) ? 1 : 0;
    *day = (UCHAR)(mjd - 14956 - (int)(yy * 365.25) - (int)(mm * 30.6001));
    *month = (UCHAR)(mm - 1 - k * 12);
    *year = (USHORT)(1900 + yy + k);
    return 0;
}

char *mjdToString(ULONG time)
{
	static char tmpStr[0x20];
	USHORT year;
	UCHAR month, day;
	int hour, min, sec=0;

    extract_mjd((USHORT)(time >> 16), &day, &month, &year);
	hour = ((time >> 8) & 0xFF);
	min = (time & 0xFF);

	memset(tmpStr, 0x0, 20);
	sprintf(tmpStr, "%02d:%02d %02d-%02d-%04d", hour, min, day, month, year);
	return tmpStr;
}

int main(int argc, char **argv)
{
    int i,j;
    char ifilename[0x200] = "test.rec";
    char ofilename[0x200] = "test.ts";
    char cfilename[0x200] = "rec.csv";
    FILE *Ip;
    FILE *Op;
    int fileno;
    int flen;
    UCHAR *ts_buf;
    int ts_len;
    type_rec_header rec;
    int len;
    UCHAR tempch;
    int ret;

    int append = 1;
    int keep = 1;

    j = 0;
    for(i=1; i<argc; i++) {
        if ((argv[i][0] == '-') || (argv[i][0] == '/')) {
            switch (argv[i][1]) {
                case 'a' :
                    append = 0;
                    break;
                case 'c' :
                    sscanf(argv[++i],"%s", cfilename);
                    j = 3;
                    break;
                case 'k' :
                    keep = 0;
                    break;
                case 'i' :
                    sscanf(argv[++i],"%s", ifilename);
                    j = 1;
                    break;
                case 'o' :
                    sscanf(argv[++i],"%s", ofilename);
                    j = 2;
                    break;
                case 'h' :
                case '?' :
                default  :
                    printf("\nrec2ts v0.01");
                    printf("\n============");
                    printf("\n");
                    printf("\nusage: rec2ts <options> <input file> <output file> <.csv file> ");
                    printf("\n         -i    input filename");
                    printf("\n         -o    output filename");
                    printf("\n         -c    .csv filename");
                    printf("\n         -a    overwrite output header info (normally appends)");
                    printf("\n         -k    delete .rec file (normally keeps)");
                    return 1;
            }
        }
        else {
            if (j == 0) {
                sscanf(argv[i],"%s", ifilename);
                j++;
            }
            else
            if (j == 1) {
                sscanf(argv[i],"%s", ofilename);
                j++;
            }
            else
            if (j == 2) {
                sscanf(argv[i],"%s", cfilename);
                j++;
            }
        }
    }

    //create a ts buffer
    ts_len = 0x100000;  //1MByte
    ts_buf = (UCHAR *)malloc(ts_len);
    if (!ts_buf) {
        printf("\nerror: ts_buf=%x",ts_buf);
        return -1;
    }

    //open input file
    printf("\nReading %s", ifilename);
    Ip = fopen(ifilename, "rb");
    if (!Ip) {
        printf("\nerror: cannot open %s", ifilename);
        free(ts_buf);
        return -1;
    }
    fileno = _fileno(Ip);
    flen = _filelength(fileno);
    printf("\nflen=0x%x, %d", flen, flen);

    //open output file
    Op = fopen(ofilename, "wb");
    if (!Op) {
        printf("\nerror: cannot open %s", ofilename);
        fclose(Ip);
        free(ts_buf);
        return -1;
    }

    //read the .rec header
    printf("\nreading header ...");
    fseek(Ip, 0, SEEK_SET);
    len = fread(&rec, 1, sizeof(type_rec_header), Ip);

    printf("\nprocessing file ...");
    fseek(Ip, 0xeb0, SEEK_SET);
    i = 0;
    do {
        len = fread(&ts_buf[0], 1, ts_len, Ip);
        fwrite(&ts_buf[0], 1, len, Op);
        i++;
        if ((i & 0xf) == 0) printf(".");        //every 16Mbytes
    } while (len);
    printf(" done");

    fclose(Ip);
    fclose(Op);
    free(ts_buf);
    if (keep = 0) {
        printf("\ndeleting %s", ifilename);
        remove(ifilename);
    }


    //now output the header info in a .csv file
    Op = fopen(cfilename, "rb");
    if (!Op) {
        ret = 1;
    }
    else {
        ret = 0;
        fclose(Op);
    }

    if ((ret == 1) || (append == 0)) {
        printf("\ncreating %s", cfilename);
        Op = fopen(cfilename, "wb");
        if (!Op) {
            printf("\nerror: cannot open %s", cfilename);
            return -1;
        }
        fprintf(Op, "filename, ");
        fprintf(Op, "lcn, ");
        fprintf(Op, "channel, ");
        fprintf(Op, "type, ");
        fprintf(Op, "starts, ");
        fprintf(Op, "ends, ");
        fprintf(Op, "duration, ");
        fprintf(Op, "program, ");
        fprintf(Op, "original_network_id, ");
        fprintf(Op, "transport_stream_id, ");
        fprintf(Op, "service_id, ");
        fprintf(Op, "event_id, ");
        fprintf(Op, "video_pid, ");
        fprintf(Op, "audio_pid, ");
        fprintf(Op, "pcr_pid, ");
//        fprintf(Op, "subtitles_pid, ");
        fprintf(Op, "pmt_pid, ");
        fprintf(Op, "description, ");
        fprintf(Op, "\n");
        fclose(Op);
    }
    printf("\nappending %s", cfilename);
    Op = fopen(cfilename, "ab");
    if (!Op) {
        printf("\nerror: cannot open %s", cfilename);
        return -1;
    }
    fprintf(Op, "%s, ", ifilename);
    fprintf(Op, "%d, ", rec.svcNum);
    fprintf(Op, "%s, ", rec.channel_name);
    fprintf(Op, "%s, ", (rec.svcType==0)?"TV":(rec.svcType==1)?"Radio":"???");
    fprintf(Op, "%s, ", mjdToString(bitrev32(rec.startTime)));
    fprintf(Op, "%s, ", mjdToString(bitrev32(rec.endTime)));
    fprintf(Op, "%02d:%02d, ", bitrev16(rec.fduration)/60, bitrev16(rec.fduration)%60);        //actual duration
//    fprintf(Op, "%02x:%02x, ", bitrev16(rec.duration)>>8, bitrev16(rec.duration)&0xff);  //epg duration
    tempch = rec.eventName_description[rec.eventName_length];
    rec.eventName_description[rec.eventName_length] = '\0';
    fprintf(Op, "%s, ", &rec.eventName_description[0]);
    fprintf(Op, "%04x, ", bitrev16(rec.original_network_id));
    fprintf(Op, "%04x, ", bitrev16(rec.transport_stream_id));
    fprintf(Op, "%04x, ", bitrev16(rec.service_id));
    fprintf(Op, "%04x, ", bitrev16(rec.event_id));
    fprintf(Op, "%04x, ", bitrev16(rec.video_pid));
    fprintf(Op, "%04x, ", bitrev16(rec.audio_pid));
    fprintf(Op, "%04x, ", bitrev16(rec.pcr_pid));
//    fprintf(Op, "%04x, ", bitrev16(rec.subtitles_pid));
    fprintf(Op, "%04x, ", bitrev16(rec.pmt_pid));
    rec.eventName_description[rec.eventName_length] = tempch;
//do last, as commas in the description mess up the .csv file
    fprintf(Op, "%s, ", &rec.eventName_description[rec.eventName_length]);
    fprintf(Op, "\n");
    fclose(Op);

    printf("\nall done");
    return 0;
}
